/*
 * CompilationUnit.java 1.0
 *
 * This subclass of symbol represents (at least) terminal symbols returned 
 * by the scanner and placed on the parse stack.  At present, this 
 * class does nothing more than its super class.
 *
 * Jun 11, 1997 by mich
 * Sep 27, 1997 by mich
 *   
 * @see openjava.ptree.Ptree
 * @version 1.0  last updated: Sep 27, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.ptree.*;


/**
 * The CompilationUnit class presents for the whole parse tree in a file.
 *
 * CompilationUnits consists of
 * (package statement) (import statement list) (type declaration list)
 * QualifiedName       ImportStatementList     TypeDeclarationList
 * 
 * @see openjava.ptree.QualifiedName
 * @see openjava.ptree.ImportStatementList
 * @see openjava.ptree.TypeDeclarationList
 */
public class CompilationUnit extends NonLeaf
{
  /**
   * Allocates this object with specified parse-tree elements.
   *
   */
  public CompilationUnit(
      QualifiedName e0,
      ImportStatementList e1,
      TypeDeclarationList e2 )
  {
    super();
    if(e1 == null)  e1 = new ImportStatementList();
    if(e2 == null)  e2 = new TypeDeclarationList();
    set( e0, e1, e2 );
  }

  /**
   * Overrides the writing code method
   *
   **/
  public void writeCode()
  {
    out.println( "/*" );
    out.println( " * This code was generated by OpenJava System." );
    out.println( " */" );

    //package statement
    QualifiedName qn = getPackage();
    if(qn != null){
      writeDebugL();
      out.print( "package " ); qn.writeCode(); out.print( ";" );
      writeDebugR(); out.println();
      
      out.println();
      out.println();
    }

    //import statement list
    ImportStatementList islst = getImportStatements();
    if(! islst.isEmpty()){
      islst.writeCode();
      
      out.println();
      out.println();
    }

    //type declaration list
    TypeDeclarationList tdlst = getTypeDeclarations();
    if(! tdlst.isEmpty()){
      tdlst.writeCode();

      out.println();
      out.println();
    }
  }

  /**
   * Sets the package of this compilation unit
   *
   * @param  qn  the qualified name indicating this package
   */
  public void setPackage( QualifiedName qn )
  {
    setElementAt( qn, 0 );
  }

  /**
   * Obtains the package of this compilation unit
   *
   * @return  the qualified name indicating this package
   */
  public QualifiedName getPackage()
  {
    return (QualifiedName) elementAt( 0 );
  }

  /**
   * Sets the import statement list of this compilation unit
   *
   * @param  islst  the import statement list of this compilation unit
   */
  public void setImportStatements( ImportStatementList islst )
  {
    setElementAt( islst, 1 );
  }

  /**
   * Obtains the import statement list of this compilation unit
   *
   * @return  the import statement list of this compilation unit
   */
  public ImportStatementList getImportStatements()
  {
    return (ImportStatementList) elementAt( 1 );
  }

  /**
   * Sets the type declaration list of this compilation unit
   *
   * @param  tdlst  the type declaration list of this compilation unit
   */
  public void setTypeDeclarations( TypeDeclarationList tdlst )
  {
    setElementAt( tdlst, 2 );
  }

  /**
   * Obtains the type declaration list of this compilation unit
   *
   * @return  the type declaration list of this compilation unit
   */
  public TypeDeclarationList getTypeDeclarations()
  {
    return (TypeDeclarationList) elementAt( 2 );
  }

  /**
   * Obtains the public class in this compilation unit.
   *
   * @return  the public class
   * @exception  PtreeException  if not one public class is declared.
   */
  public TypeDeclaration getPublicClass()
    throws PtreeException
  {
    TypeDeclaration ret = null;

    TypeDeclarationList tdecls = getTypeDeclarations();
    int len = tdecls.getLength();
    for( int i = 0; i < len; i++ ){
      TypeDeclaration tdecl = tdecls.elementAt( i );
      if(tdecl instanceof ClassDeclaration){
	ClassDeclaration cdecl = (ClassDeclaration) tdecl;
	if(cdecl.getModifiers().contains( "public" )){
	  if(ret != null){
	    throw new PtreeException( "getPublicClass() " +
				      "in CompileationUnit : " +
				      "multiple public class" );
	  }
	  ret = cdecl;
	}
      }else if(tdecl instanceof InterfaceDeclaration){
	InterfaceDeclaration idecl = (InterfaceDeclaration) tdecl;
	if(idecl.getModifiers().contains( "public" )){
	  if(ret != null){
	    throw new PtreeException( "getPublicClass() " +
				      "in CompileationUnit : " +
				      "multiple public class" );
	  }
	  ret = idecl;
	}
      }
    }
    
    return ret;
  }

}
