/*
 * ConstructExpression.java 1.0
 *
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>ConstructExpression</code> class represents
 * an expression which allocates a new object with its constructor.
 * <br>
 * This expression is like:
 * <br><blockquote><pre>
 *     new String( "test" )
 * </pre></blockquote><br>
 * or:
 * <br><blockquote><pre>
 *     new String( "test" ){
 *             public void hoge(){ ... }
 *             ...
 *         }
 * </pre></blockquote><br>
 * The latter is supported from JDK 1.1, is called an anoymous class
 * of the inner class.
 * 
 * @see openjava.ptree.Expression
 * @see openjava.ptree.ClassType
 * @see openjava.ptree.ExpressionList
 * @see openjava.ptree.MemberDeclarationList
 */
public class ConstructExpression extends NonLeaf
    implements Expression
{
  /**
   * Allocates a new object with the class body.
   *
   * @param  ctype  a class name to be constructed
   * @param  args  argument list of constructor
   * @param  mdlst  the class body.
   *                If this is null, no class body will be provided
   *                this construct expression with.
   */
  public ConstructExpression(
      ClassType              typename,
      ExpressionList         args,
      MemberDeclarationList  mdlst )
  {
    super();
    if(args == null){
      args = new ExpressionList();
    }
    set( typename, args, mdlst );
  }
  
  /**
   * Allocates a new object without class body.
   *
   * @param  ctype  a class name to be constructed
   * @param  args  argument list of constructor
   */
  public ConstructExpression(
      ClassType       ctype,
      ExpressionList  args )
  {
    super();
    if(args == null){
      args = new ExpressionList();
    }
    set( (Ptree) ctype, (Ptree) args, null );
  }

  ConstructExpression()
  {
    super();
  }

  public void writeCode()
  {
    writeDebugL();

    out.print( "new " );
    
    ClassType tn = getClassType();
    tn.writeCode();
    
    out.print( "(" );
    ExpressionList el = getArguments();
    if(! el.isEmpty()){
      out.print(" ");
      el.writeCode();
      out.print(" ");
    }else{
      el.writeCode();
    }
    out.print( ")" );

    MemberDeclarationList mdlst = getClassBody();
    if(mdlst != null){
      out.println( "{" );
      pushNest();  mdlst.writeCode();  popNest();
      writeTab();  out.print( "}" );
    }

    writeDebugR();
  }
  
  /**
   * Gets the class type of this constructor.
   *
   * @return  the class type of this constructor.
   */
  public ClassType getClassType()
  {
    return (ClassType) elementAt( 0 );
  }

  /**
   * Sets the class type of this constructor.
   *
   * @param  ctype  the class body to set.
   */
  public void setClassType( ClassType ctype )
  {
    setElementAt( ctype, 0 );
  }
  
  /**
   * Gets the arguments of this constructor.
   *
   * @return  the arguments as an expressions list.
   */
  public ExpressionList getArguments()
  {
    return (ExpressionList) elementAt( 1 );
  }

  /**
   * Sets the arguments of this constructor.
   *
   * @return  the expressions list of arguments.
   */
  public void setArguments( ExpressionList args )
  {
    if(args == null){
      args = new ExpressionList();
    }
    setElementAt( args, 1 );
  }

  /**
   * Gets the class body of this constructor.
   *
   * @return  the member declaration list as the class body of
   *          this constructor.
   */
  public MemberDeclarationList getClassBody()
  {
    return (MemberDeclarationList) elementAt(2);
  }

  /**
   * Sets the class body of this constructor.
   *
   * @param  mdlist  the member declaration list of the class body.
   *                 If this is null, the class body will disappear.
   */
  public void setClassBody( MemberDeclarationList mdlist )
  {
    setElementAt( mdlist, 2 );
  }

}
