/*
 * ConstructorDeclaration.java 1.0
 *
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import openjava.util.PartialParser;


/**
 * The <code>ConstructorDeclaration</code> class represents
 * constructor declaration node of the parse tree.
 *
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.MemberDeclaration
 * @see openjava.ptree.ModifierList
 * @see openjava.ptree.Identifier
 * @see openjava.ptree.ParameterList
 * @see openjava.ptree.ClassTypeList
 * @see openjava.ptree.SpecialCallStatement
 * @see openjava.ptree.StatementList
 */
public class ConstructorDeclaration extends NonLeaf
    implements MemberDeclaration
{
  /**
   * Constructs new ConstructorDeclaration from its elements.
   * 
   * @param  modiflist  modifier list, if it has no modifier list
   *                    then thes arg is set empty list.
   * @param  name  name of this constructor.
   * @param  params  parameter list
   * @param  throwlist  throw type list, if there is no throws
   *                    then this arg is set empty list
   * @param  scstmt  statement which calls another constructor
   *                 if this is null, it means no another constructor
   *                 call exists.
   * @param  stmtlist  statement list of this constructor body.
   *                   if this is null, it means method body is with
   *                   only semi colon.
   */
  public ConstructorDeclaration(
    ModifierList          modiflist,
    Identifier            name,
    ParameterList         params,
    ClassTypeList         throwlist,
    SpecialCallStatement  scstmt,
    StatementList         stmtlist )
  {
    super();
    if(modiflist == null){
      modiflist = new ModifierList();
    }
    if(params == null){
      params = new ParameterList();
    }
    if(throwlist == null){
      throwlist = new ClassTypeList();
    }
    set( modiflist, name, params, throwlist, scstmt, stmtlist );
  }

  /**
   * Constructs new ConstructorDeclaration from its elements.
   * 
   * @param  modiflist  modifier list, if it has no modifier list
   *                    then thes arg is set empty list.
   * @param  name  name of this constructor.
   * @param  params  parameter list
   * @param  throwlist  throw type list, if there is no throws
   *                    then this arg is set empty list
   * @param  stmtlist  statement list of this constructor body.
   *                   if this is null, it means method body is with
   *                   only semi colon.
   */
  public ConstructorDeclaration(
    ModifierList          modiflist,
    Identifier            name,
    ParameterList         params,
    ClassTypeList         throwlist,
    StatementList         stmtlist )
  {
    this( modiflist, name, params, throwlist, null, stmtlist );
  }

  ConstructorDeclaration()
  {
    super();
  }

  public void writeCode()
  {
    openjava.util.Comment comment = getComment();
    if(comment != null)  comment.writeCode();

    writeTab();  writeDebugL();

    ModifierList ml = getModifiers();
    ml.writeCode();
    if(! ml.isEmpty()){
      out.print( " " );
    }

    Identifier name = getName();
    name.writeCode();

    out.print( "(" );

    ParameterList params = getParameters();
    if(params.getLength() != 0){
      out.print( " " );
      params.writeCode();
      out.print( " " );
    }

    out.print( ")" );
    
    ClassTypeList tnl = getThrows();
    if(tnl != null && !tnl.isEmpty()){
      out.println();
      out.print( "throws " );
      tnl.writeCode();
    }

    StatementList body = getBody();
    SpecialCallStatement sc = getSpecialCallStatement();
    if(body == null && sc == null){
      out.println( ";" );
    }else{
      out.println();

      writeTab();  out.println( "{" );
      pushNest();

      if(sc != null){
	sc.writeCode();
      }

      body.writeCode();

      popNest();

      writeTab();  out.print( "}" );
    }

    writeDebugR();  out.println();
  }
  
  /**
   * Gets modifier list.
   *
   * @return modifier list.
   */
  public ModifierList getModifiers()
  {
    return (ModifierList) elementAt( 0 );
  }

  /**
   * Sets modifier list.
   *
   * @param  modifs  modifier list.
   */
  public void setModifiers( ModifierList modifs )
  {
    if(modifs == null){
      modifs = new ModifierList();
    }
    setElementAt( modifs, 0 );
  }
  
  /**
   * Gets the name of this constructor node.
   *
   * @return constructor declarator node.
   */
  public Identifier getName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * Sets the name of this constructor node.
   *
   * @param  name  the name to be set.
   */
  public void setName( Identifier name )
  {
    setElementAt( name, 1 );
  }

  /**
   * Gets the parameter list.
   *
   * @return parameter list for constructor.
   */
  public ParameterList getParameters()
  {
    return (ParameterList) elementAt( 2 );
  }

  /**
   * Sets the parameter list.
   *
   * @param  params  parameterlist for constructor declarator node.
   */
  public void setParameters( ParameterList params )
  {
    if(params == null){
      params = new ParameterList();
    }
    setElementAt( params, 2 );
  }
  
  /**
   * Gets the class type list thrown by this constructor.
   *
   * @return class type list thrown by this constructor.
   */
  public ClassTypeList getThrows()
  {
    return (ClassTypeList) elementAt( 3 );
  }

  /**
   * Sets the class type list thrown by this constructor.
   *
   * @param  ctlist  class type list thrown by this constructor.
   */
  public void setThrows( ClassTypeList ctlist )
  {
    if(ctlist == null){
      ctlist = new ClassTypeList();
    }
    setElementAt( ctlist, 3 );
  }
  
  /**
   * Gets the special call statement.
   * Special call statement is like:
   * <br><blockquote><pre>
   *     super();
   * </pre></blockquote><br>
   *
   * @return  special call statement
   */
  public SpecialCallStatement getSpecialCallStatement()
  {    
    return (SpecialCallStatement) elementAt( 4 );
  }

  /**
   * Sets the special call statement.
   *
   * @param  scstmt  the special call statement to set
   * @see openjava.ptree.ConstructorDeclaration#getSpecialCallStatement()
   */
  public void setSpecialCallStatement( SpecialCallStatement scstmt)
  {    
    setElementAt( scstmt, 4 );
  }

  /**
   * Gets the statement list of this constructor body.
   *
   * @return  the statement list of this constructor body.
   */
  public StatementList getBody()
  {    
    return (StatementList) elementAt( 5 );
  }

  /**
   * Sets the statement list of this constructor body.
   *
   * @return  the statement list of this constructor body.
   */
  public void setBody( StatementList stmts )
  {    
    setElementAt( stmts, 5 );
  }

  public static ConstructorDeclaration make( String str )
  {
    ConstructorDeclaration ret = null;

    try{
      ret = (ConstructorDeclaration)
	PartialParser.makeMemberDeclaration( str );
    }catch( Exception ex ){
      return null;
    }

    return ret;
  }

}

