/*
 * ForStatement.java 1.0
 *
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>ForStatement</code> class represents a for statement
 * node of parse tree.
 * <br>
 * The specification of the initialization part may be modified
 * in the later version of OpenJava.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.Statement
 * @see openjava.ptree.Expression
 * @see openjava.ptree.ExpressionList
 * @see openjava.ptree.StatementList
 */
public class ForStatement extends NonLeaf
    implements Statement, Ptree
{
  /**
   * Allocates a new ForStatement object.
   *
   */
  public ForStatement()
  {
    super();
    set( null, null, null, new StatementList() );
  }

  /**
   * Allocates a new ForStatement object.
   *
   * @param  init  the initialization part.
   * @param  expr  the condition part.
   * @param  iterator  the increments part.
   * @param  stmts  the stement list of the body.
   */
  public ForStatement(
      ForInit         init,
      Expression      expr,
      ExpressionList  itarator,
      StatementList   stmts )
  {
    super();
    set( init, (Ptree) expr, itarator, (Ptree) stmts );
  }

  /**
   * Allocates a new ForStatement object.
   *
   * @param  init  the initialization part.
   * @param  expr  the condition part.
   * @param  iterator  the increments part.
   * @param  stmts  the stement list of the body.
   */
  public ForStatement(
      ExpressionList  init,
      Expression      expr,
      ExpressionList  iterator,
      StatementList   stmts )
  {
    super();
    set( init, (Ptree) expr, iterator, (Ptree) stmts );
  }

  /**
   * The <code>ForInit</code> inner-class presents for-initialize node
   * of parse tree
   *
   * @see openjava.ptree.Ptree
   * @see openjava.ptree.NonLeaf
   */
  public class ForInit extends List
  {
    private TypeSpecifier tspec = null;
    /**
     * Constructs a new ForInit object
     *
     * @param inti prototype object
     */
    public ForInit( TypeSpecifier tspec )
    {
      super( ", " );
      this.tspec = tspec;
    }

    public void writeCode()
    {
      TypeSpecifier tspec = getTypeSpecifier();
      tspec.writeCode();
      out.print( " " );

      super.writeCode();
    }

    public void addElement( VariableDeclarator vdecl )
    {
      ptreelist.addElement( vdecl );
    }

    public VariableDeclarator elementAt( int n )
    {
      return (VariableDeclarator) ptreelist.elementAt( n );
    }
    
    public TypeSpecifier getTypeSpecifier()
    {
      return this.tspec;
    }

  }
  
  public void writeCode()
  {
    writeTab();  writeDebugL();

    out.print( "for" );
    out.print( "( " );
    
    List init = getInit();
    init.writeCode();

    out.print( "; " );
    
    Expression expr = getCondition();
    if(expr != null){
      expr.writeCode();
    }

    out.print( "; " );
    
    ExpressionList incr = getIncrement();
    incr.writeCode();
    
    out.print( " )" );
    
    StatementList stmts = getStatements();
    if(stmts.isEmpty()){
      out.print( " ;" );
    }else{
      out.println( "{" );
      pushNest();  stmts.writeCode();  popNest();
      writeTab();  out.print( "}" );
    }

    writeDebugR();  out.println();
  }

  /**
   * Gets the initialization part of this for-statement.
   *
   * @return  the initialization part.
   */
  public List getInit()
  {
    return (List) elementAt( 0 );
  }

  /**
   * Sets the initialization part of this for-statement.
   *
   * @param  init  the initialization part.
   */
  public void setInit( ForInit init )
  {
    setElementAt( init, 0 );
  }

  /**
   * Sets the initialization part of this for-statement.
   *
   * @param  init  the initialization part.
   */
  public void setInit( ExpressionList init )
  {
    setElementAt( init, 0 );
  }
  
  /**
   * Gets the condition part of this for-statement.
   *
   * @return  the expression of the condtion part.
   */
  public Expression getCondition()
  {
    return (Expression) elementAt( 1 );
  }

  /**
   * Sets the condition part of this for-statement.
   *
   * @param  cond  the expression of the condtion part.
   */
  public void setCondition( Expression cond )
  {
    setElementAt( cond, 1 );
  }
  
  /**
   * Gets the increments part of this for-statement.
   *
   * @return  the expression list of the increments part.
   */
  public ExpressionList getIncrement()
  {
    return (ExpressionList) elementAt( 2 );
  }

  /**
   * Sets the increments part of this for-statement.
   *
   * @param  incs  the expression list of the increments part.
   */
  public void setIncrement( ExpressionList incs )
  {
    setElementAt( incs, 2 );
  }
  
  /**
   * Gets the body of this for-statement.
   *
   * @return  the statement list of the body.
   */
  public StatementList getStatements()
  {
    return (StatementList) elementAt( 3 );
  }

  /**
   * Sets the body of this for-statement.
   *
   * @param  stmts  the statement list of the body.
   */
  public void setStatements( StatementList stmts )
  {
    setElementAt( stmts, 3 );
  }

}
