/*
 * NonLeaf.java 1.0
 *
 * This subclass of symbol represents (at least) terminal symbols returned 
 * by the scanner and placed on the parse stack.  At present, this 
 * class does nothing more than its super class.
 * 
 * Jun 11, 1997
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.PtreeObject
 * @version 1.0 last updated: Jun 11, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import java.io.Writer;
import java.io.PrintWriter;
import java.io.StringWriter;
import openjava.tools.StringBufferInputStream;
import openjava.main.Parser;
import openjava.util.Comment;

/**
 * The NonLeaf class presents for node of parse tree.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.Leaf
 */
public class NonLeaf extends PtreeObject
implements Ptree
{
  /** contents is the array of the elements of this nonleaf-node. */
  private Ptree[] contents = null;

  /**
   * Allocates a new non-leaf(cons-cell), where the first element has
   * null and the second element has null.
   * 
   */
  public NonLeaf() {
    contents = new Ptree[0];
  }

  /**
   * Makes a new copy (another object) of this nonleaf-node recursively.
   * The objects contained by this object will also be copied.
   *
   * @return  the copy of this nonleaf-node as a ptree-node.
   */
  public Ptree makeRecursiveCopy()
  {
    Ptree newc[] = new Ptree[contents.length];
    
    for( int i = 0; i < contents.length; i++ ){
      newc[i] = null;
      if(contents[i] != null){
	newc[i] = contents[i].makeRecursiveCopy();
      }
    }

    NonLeaf ret = null;
    try{
      ret = (NonLeaf) this.getClass().newInstance();
    }catch( InstantiationException ex ){
      System.err.println( "makeCopy() error : " + ex );
      return null;
    }catch( IllegalAccessException ex ){
      System.err.println( "makeCopy() error : " + ex );
      return null;
    }

    ret.set( newc );

    Comment comment = getComment();
    if (comment != null) {
      ret.setComment( (Comment) comment.makeRecursiveCopy() );
    }

    return ret;
  }

  /**
   * Makes a new copy of this nonleaf-node as a ptree-node.
   * The objects contained by the new object are same as
   * these contained by the original object.
   *
   * @return  the copy of this nonleaf-node as a ptree-node.
   */
  public Ptree makeCopy()
  {
    Ptree newc[] = new Ptree[contents.length];
    
    for( int i = 0; i < contents.length; i++ ){
      newc[i] = contents[i];
    }

    NonLeaf ret = null;
    try{
      ret = (NonLeaf) this.getClass().newInstance();
    }catch( InstantiationException ex ){
      System.err.println( "makeCopy() error : " + ex );
      return null;
    }catch( IllegalAccessException ex ){
      System.err.println( "makeCopy() error : " + ex );
      return null;
    }
    ret.set( newc );
    ret.setComment( getComment() );

    return ret;
  }

  /**
   * Tests if this nonleaf-node's value equals to the specified
   * ptree-node's.
   *
   * @return  true if two values are same.
   */
  public boolean equals( Ptree p )
  {
    if(p == null)  return false;
    if(eq( this, p ))  return true;
    if(this.getClass() != p.getClass())  return false;

    NonLeaf nlp = (NonLeaf) p;
    int len = this.getLength();
    if(len != nlp.getLength())  return false;
    for( int i = 0; i < len; i++ ){
      if(! equal( this.elementAt( i ), nlp.elementAt( i ) ))
	return false;
    }
    return true;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p ]
   *
   * @param  p  list's element
   */
  protected void set( Ptree[] ptrees )
  {
    contents = ptrees;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p ]
   *
   * @param  p  list's element
   */
  protected void set( Ptree p )
  {
    contents = new Ptree[1];
    contents[0] = p;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 ]
   *
   * @param  p0  list's element
   * @param  p1  list's element
   */
  protected void set( Ptree p0, Ptree p1 )
  {
    contents = new Ptree[2];
    contents[0] = p0;
    contents[1] = p1;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 p2 ]
   *
   * @param  p0  list's element
   * @param  p1  list's element
   * @param  p2  list's element
   */
  protected void set( Ptree p0, Ptree p1, Ptree p2 )
  {
    contents = new Ptree[3];
    contents[0] = p0;
    contents[1] = p1;
    contents[2] = p2;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 p2 p3 ]
   *
   * @param p0 list's element
   * @param p1 list's element
   * @param p2 list's element
   * @param p3 list's element
   */
  protected void set( Ptree p0, Ptree p1, Ptree p2, Ptree p3 )
  {
    contents = new Ptree[4];
    contents[0] = p0;
    contents[1] = p1;
    contents[2] = p2;
    contents[3] = p3;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 p2 p3 p4 ]
   *
   * @param p0 list's element
   * @param p1 list's element
   * @param p2 list's element
   * @param p3 list's element
   * @param p4 list's element
   */
  protected void set( Ptree p0, Ptree p1, Ptree p2, Ptree p3,
		      Ptree p4 )
  {
    contents = new Ptree[5];
    contents[0] = p0;
    contents[1] = p1;
    contents[2] = p2;
    contents[3] = p3;
    contents[4] = p4;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 p2 p3 p4 p5 ]
   *
   * @param p0 list's element
   * @param p1 list's element
   * @param p2 list's element
   * @param p3 list's element
   * @param p4 list's element
   */
  protected void set( Ptree p0, Ptree p1, Ptree p2, Ptree p3,
		      Ptree p4, Ptree p5 )
  {
    contents = new Ptree[6];
    contents[0] = p0;
    contents[1] = p1;
    contents[2] = p2;
    contents[3] = p3;
    contents[4] = p4;
    contents[5] = p5;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 p2 p3 p4 p5 p6 ]
   *
   * @param p0 list's element
   * @param p1 list's element
   * @param p2 list's element
   * @param p3 list's element
   * @param p4 list's element
   * @param p5 list's element
   * @param p6 list's element
   */
  protected void set( Ptree p0, Ptree p1, Ptree p2, Ptree p3,
		      Ptree p4, Ptree p5, Ptree p6 )
  {
    contents = new Ptree[7];
    contents[0] = p0;
    contents[1] = p1;
    contents[2] = p2;
    contents[3] = p3;
    contents[4] = p4;
    contents[5] = p5;
    contents[6] = p6;
  }

  /**
   * Makes this ptree a list presenting for
   * [ p0 p1 p2 p3 p4 p5 p6 p7 ]
   *
   * @param p0 list's element
   * @param p1 list's element
   * @param p2 list's element
   * @param p3 list's element
   * @param p4 list's element
   * @param p5 list's element
   * @param p6 list's element
   * @param p7 list's element
   */
  protected void set( Ptree p0, Ptree p1, Ptree p2, Ptree p3,
		      Ptree p4, Ptree p5, Ptree p6, Ptree p7 )
  {
    contents = new Ptree[8];
    contents[0] = p0;
    contents[1] = p1;
    contents[2] = p2;
    contents[3] = p3;
    contents[4] = p4;
    contents[5] = p5;
    contents[6] = p6;
    contents[7] = p7;
  }

  /**
   * Writes the code this parse-tree presents for.
   *
   */
  public void writeCode()
  {
    writeDebugL();

    if(contents.length == 0){
      System.err.println( "writeCode() error : no contents." );
      //throw new PtreeException( "writeCode() error : no contents." );
    }else{
      contents[0].writeCode();
    }
    for( int i = 1; i < contents.length; i++ ){
      out.print( " " );
      writeCode();
    }

    writeDebugR();
  }

  /**
   * Returns the specified element at the specified point of this
   * nonleaf-node.
   *
   * @param  i  index
   * @return  the ptree object at the specified point
   */
  protected Ptree elementAt( int i )
  {
    Ptree ret = null;

    try{
      ret = contents[i];
    }catch( ArrayIndexOutOfBoundsException ex ){
      System.out.println( ex );
      //throw new NonLeafException( ex.toString() );
    }

    return ret;
  }

  /**
   * Sets the specified element at the specified point of this
   * nonleaf-node.
   *
   * @param  p  ptree object to set
   * @param  i  index
   */
  protected void setElementAt( Ptree p, int i )
  {
    try{
      contents[i] = p;
    }catch( ArrayIndexOutOfBoundsException ex ){
      System.out.println( ex );
      //throw new NonLeafException( ex.toString() );
    }
  }

  /**
   * Gets the contents of this nonleaf-node.
   *
   * @return  contents
   */
  protected Ptree[] getContents()
  {
     return contents;
  }

  /**
   * getLength() returns the length of this nonleaf-node
   *
   * @return the length of the list
   */
  protected int getLength()
  {
    return contents.length;
  }

  private openjava.util.Comment comment = null;

  /**
   * Sets the comment of javadoc format which explains this declaration.
   *
   * @param  comment  the Comment object to set
   */
  public void setComment( openjava.util.Comment comment )
  {
    this.comment = comment;
  }

  /**
   * Obtains the comment of javadoc format which explains this declaration.
   *
   * @return  the string of the comment
   */
  public openjava.util.Comment getComment()
  {
    return this.comment;
  }

}
