/*
 * PartialParser.java 1.0
 *
 * This class can be used to make ptree objects from string.
 * 
 * Jun 11, 1997 mich
 * Oct 17, 1997 mich
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.PtreeObject
 * @version 1.0 last updated: Oct 17, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.util;


import java.lang.String;
import java.io.StringWriter;
import openjava.tools.StringBufferInputStream;
import openjava.main.Parser;
import openjava.ptree.*;

/**
 * The <code>PartialParser</code> class is
 * an utilty class to make ptree objects from string.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.Expression
 * @see openjava.ptree.Statement
 * @see openjava.ptree.StatementList
 * @see openjava.ptree.MemberDeclaration
 * @see openjava.ptree.MemberDeclarationList
 * @see openjava.ptree.TypeDeclaration
 * @see openjava.ptree.CompilationUnit
 */
public class PartialParser
{
  /**
   * Constructor should not be called.
   *
   */
  protected PartialParser()
  {
  }

  /**
   * Makes a ptree node from the string like :
   * <br><blockquote><pre>
   *     "i + 3"
   * </pre></blockquote><br>
   * or :
   * <br><blockquote><pre>
   *     "f()"
   * </pre></blockquote><br>
   *
   * @return  the expression node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static Expression makeExpression( String str )
    throws PtreeException
  {
    Expression ret = null;

    try{
      String pstr = "int i=" + str + ";";

      FieldDeclaration field
	= (FieldDeclaration) makeMemberDeclaration( pstr );

      VariableInitializer vinit = field.getInitializer();
      if(! (vinit instanceof Expression)){
	throw new PtreeException(
	    "syntax error" );
      }

      ret = (Expression) vinit;
    }catch( PtreeException ex1 ){
      throw ex1;
    }catch( Exception ex2 ){
      throw new PtreeException(
	  "syntax error" );
    }    

    return ret;
  }

  /**
   * Makes a ptree node from the string like :
   * <br><blockquote><pre>
   *     "i++;"
   * </pre></blockquote><br>
   * or :
   * <br><blockquote><pre>
   *     "for(;;){ f(); }"
   * </pre></blockquote><br>
   *
   * @return  the statement node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static Statement makeStatement( String str )
    throws PtreeException
  {
    StatementList stmts = null;
    Statement ret = null;

    try{
      stmts = makeStatementList( str );
      if(stmts.getLength() != 1){
	throw new PtreeException(
	    "syntax error: Not a single statement." );
      }

      ret = stmts.elementAt( 0 );
    }catch( Exception ex ){
      throw new PtreeException(
	  "syntax error" );
    }    

    return ret;
  }

  /**
   * Makes ptree node from the string like :
   * <br><blockquote><pre>
   *     "i++; j = 3;"
   * </pre></blockquote><br>
   *
   * @return  the statements node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static StatementList makeStatementList( String str )
    throws PtreeException
  {
    StatementList ret = null;

    try{
      String pstr = "int f(){" + str + "}";

      MethodDeclaration mtddecl
	= (MethodDeclaration) makeMemberDeclaration( pstr );

      ret = mtddecl.getBody();
    }catch( PtreeException ex1 ){
      throw ex1;
    }catch( Exception ex2 ){
      throw new PtreeException(
	  "syntax error" );
    }    

    return ret;
  }

  /**
   * Makes ptree node from the string like :
   * <br><blockquote><pre>
   *     "int f(){ return 1; }"
   * </pre></blockquote><br>
   * or :
   * <br><blockquote><pre>
   *     "public String str;"
   * </pre></blockquote><br>
   *
   * @return  the statements node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static MemberDeclaration makeMemberDeclaration( String str )
    throws PtreeException
  {
    MemberDeclaration ret = null;

    try{
      MemberDeclarationList mlist = makeMemberDeclarationList( str );
      if(mlist.getLength() != 1){
	throw new PtreeException(
	    "syntax error: Not a single member declaration." );
      }
      ret = mlist.elementAt( 0 );
    }catch( PtreeException ex1 ){
      throw ex1;
    }catch( Exception ex2 ){
      throw new PtreeException(
	  "syntax error" );
    }    

    return ret;
  }

  /**
   * Makes a ptree node from the string like :
   * <br><blockquote><pre>
   *     "int f(){ return 1; }" +
   *     "public String str;"
   * </pre></blockquote><br>
   *
   * @return  the type declarations list node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static MemberDeclarationList makeMemberDeclarationList( String str )
    throws PtreeException
  {
    MemberDeclarationList ret = null;

    try{
      String pstr = "class a{" + str + "}";
      TypeDeclaration tdecl
	= (ClassDeclaration) makeTypeDeclaration( pstr );

      ret = tdecl.getBody();
    }catch( PtreeException ex1 ){
      throw ex1;
    }catch( Exception ex2 ){
      throw new PtreeException(
	  "syntax error" );
    }    

    return ret;
  }

  /**
   * Makes a TypeDeclaration object from the string like :
   * <br><blockquote><pre>
   *     "public class Test{" +
   *     "  public String str;" +
   *     "  private int f(){ return 1; }" +
   *     "}"
   * </pre></blockquote><br>
   *
   * @return  the typedeclaration node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static TypeDeclaration makeTypeDeclaration( String str )
    throws PtreeException
  {
    TypeDeclaration ret = null;

    try{
      CompilationUnit cunit = makeCompilationUnit( str );

      QualifiedName pack = cunit.getPackage();
      ImportStatementList islist = cunit.getImportStatements();
      if(! (pack == null || islist.isEmpty())){
	throw new PtreeException(
	    "syntax error: Some package or import statements exist." );
      }
      
      TypeDeclarationList typedecls = cunit.getTypeDeclarations();
      if(typedecls.getLength() != 1){
	throw new PtreeException(
	    "syntax error: Not a single type declaration." );
      }

      ret = typedecls.elementAt( 0 );
    }catch( PtreeException ex1 ){
      throw ex1;
    }catch( Exception ex2 ){
      throw new PtreeException( ex2.toString() );
    }

    return ret;
  }

  /**
   * Makes a CompilationUnit object from the string.
   *
   * @return  the comilation unit node which the specified string
   *          represents.
   * @exception  PtreeException  if any critical error occurs.
   */
  public static CompilationUnit makeCompilationUnit( String str )
    throws PtreeException
  {
    StringBufferInputStream in = new StringBufferInputStream( str );
    Parser parser = null;
    try{
      parser = new Parser( in );
      parser.parse();
    }catch( Exception ex ){
      throw new PtreeException( ex.toString() );
    }

    CompilationUnit ret = null;

    try{
      ret = parser.getParseTree();
    }catch( Exception ex ){
      throw new PtreeException( ex.toString() );
    }

    return ret;
  }

}
